# -*- coding: utf-8 -*-
#    Copyright (C) 2005, 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
The fwbackups logger
"""

import logging
import time, datetime
from fwbackups.const import *
from fwbackups.i18n import _
from fwbackups import fwbackupsError

def _setupLogger(ui=False, level=logging.DEBUG, printToo=False):
    """ Setup the fwbackups logger, text mode """
    try:
        logging.setLoggerClass(Logger)
        # Create logger and set level to debug
        log = logging.getLogger(LOGGERNAME)
        log.setLevel(level)
        loghandler = logging.FileHandler(logloc, 'a')
        # Create formatter & add formatter to handler
        logformatter = logging.Formatter("%(message)s")
        loghandler.setFormatter(logformatter)
        # add handler to logger
        log.addHandler(loghandler)
        log.setPrintToo(printToo)
        return log, loghandler
    except Exception, e:
        raise fwbackupsError(_('Error setting up the log. Aborting. Reason:\n%s')\
                              % str(e))

class Logger(logging.Logger):
    """ A subclass to logging.Logger """
    def __init__(self, name, level=logging.INFO):
        logging.Logger.__init__(self, name, level)
        self.printToo = False
        self.function = None
        self.__newmessages = False
        
    def setPrintToo(self, printToo):
        """ Set printToo """
        if printToo == True:
            self.printToo = True
            return True
        elif printToo == False:
            self.printToo = False
            return True
        else:
            return False
    
    def getPrintToo(self):
        return self.printToo
    
    def connect(self, function):
        """ Connects the logmsg to a function """
        self.function = function

    def getNewMessages(self):
        """ Returns if there are any new messages since starting the logger """
        return self.__newMessages

    def logmsg (self, severity, message):
        """ Logs a message.
            severity: string, one of debug, info, warning, error or critical.
            message: Message to log
        """
        date = datetime.datetime.now().strftime('%b %d %H:%M:%S')
        msg = date + ": " + message
        self.__newmessages = True
        if severity.lower() == "debug":
            self.debug('%s :: %s' % (_('DEBUG'), msg))
            if self.printToo == True:
                print '%s :: %s' % (_('DEBUG'), msg)
        elif severity.lower() == "info":
            self.info('%s :: %s' % (_('INFO'), msg))
            if self.printToo == True:
                print '%s :: %s' % (_('INFO'), msg)
        elif severity.lower() == "warning":
            self.warning('%s :: %s' % (_('WARNING'), msg))
            if self.printToo == True:
                print '%s :: %s' % (_('WARNING'), msg)
        elif severity.lower() == "error":
            self.error('%s :: %s' % (_('ERROR'), msg))
            if self.printToo == True:
                print '%s :: %s' % (_('ERROR'), msg)
        elif severity.lower() == "critical":
            self.critical('%s :: %s' % (_('CRITICAL'), msg))
            if self.printToo == True:
                print '%s :: %s' % (_('CRITICAL'), msg)
        else:
            raise ValueError(_('Invalid level \'%s\' passed') % severity)
        if self.function:
            self.function(severity.lower(), msg)
            