#! /usr/bin/python
# -*- coding: utf-8 -*-
#    Copyright (C) 2005, 2006, 2007 Stewart Adam
#    This file is part of fwbackups.

#    fwbackups is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwbackups is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwbackups; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Puts it all together.
"""
import gtk
import gtk.glade
import gobject
import getopt
import os
import sys
import logging

from fwbackups.i18n import _
from fwbackups.const import *
from fwbackups import fwlogger, config, guicallbacks, guiwidgets, fwbackupsError
gobject.threads_init()

class fwbackups(guicallbacks.Callbacks):
    """ Main program """
    def __init__(self, verbose):
        guicallbacks.Callbacks.__init__(self)
        self.operationInProgress = True
        config._setupConf()
        if verbose == True:
            level = 'debug'
        else:
            level = 'info'
        try:
            self.logger, self.loghandler = self._setupLogger(level)
        except fwbackupsError, e:
            print _('Error setting up the log. Aborting.')
            print _('Reason: %s' % str(e))
            sys.exit(1)
            
        #self._setupTrayIcon()
        self.statusbar = guiwidgets.StatusBar(self.ui.statusbar1)
        # Creating the various views
        self.OneTimeView1 = guiwidgets.OneTimeView(self.ui.OneTimeTreeview, self.statusbar, self.ui)
        self.SetEditorPathView1 = guiwidgets.PathView(self.ui.BackupDiaTreeview, self.statusbar, self.ui)
        self.SetView1 = guiwidgets.SetView(self.ui.BackupSetIconview, self.statusbar, self.SetEditorPathView1, self.ui)
        self.ExportView1 = guiwidgets.ExportView(self.ui.ExportTreeview, self.statusbar, self.ui)
        # Defaults, labels...
        self._setDefaults()
        self._setLabels()
        # Parent windows...
        self.ui.about.set_transient_for(self.ui.main)
        self.ui.path_dia.set_transient_for(self.ui.main)
        self.ui.backupset_dia.set_transient_for(self.ui.main)
        self.ui.confirm_dia.set_transient_for(self.ui.main)
        self.ui.error_dia.set_transient_for(self.ui.main)
        self.ui.path_dia.set_transient_for(self.ui.main)
        self.ui.save_dia.set_transient_for(self.ui.main)
        self.ui.export_dia.set_transient_for(self.ui.main)
        self.ui.options_dia.set_transient_for(self.ui.main)
        # Log size...
        self._checkLogSize()
        
        # Welcome...
        self.logger.logmsg('INFO', _('fwbackups administrator started'))
        self.ui.WelcomeLabel.set_text(_('Welcome, %s'  % USER))
        self.operationInProgress = False

    def _checkLogSize(self):
        """ Check the log size, clear if needed """
        statsize = int(os.stat(logloc)[6])
        if statsize >= 1048577:
            #1048576 is a MB. Log will be literally bigger!
            size = '%s KB' % (statsize / 1024)
            confirmDialog = ConfirmDia(self.ui.confirm_dia, self.ui.ConfirmDiaLabel, i18n.QUESTION_LOG_SIZE_LARGE_CLEAR % size)
            response = confirmDialog.run()
            if response == gtk.RESPONSE_YES:
                self.loghandler.clear()
                self.logger.get_label().set_text(str(0))
                self.logger.logmsg('INFO', i18n.MSG_LOG_CLEARED)
            confirmDialog.destroy()

    def _setLabels(self):
        """ Set label around the program to the correct values """
        #label = self.ui.AboutDiffingoComButton.get_label()
        #label.set_text('<span color="blue">www.diffingo.com</span>')
        #label.set_use_markup(True)
        self.ui.AboutVersionLabel.set_text('<span size="xx-large" weight="bold">fwbackups %s</span>' % VERSION)
        self.ui.AboutVersionLabel.set_use_markup(True)

    def _setupTrayIcon(self):
        """ Sets up the tray icon """
        pix = self.ui.main.render_icon(gtk.STOCK_COPY, gtk.ICON_SIZE_MENU)
        #pix = gtk.gdk.pixbuf_new_from_file("/usr/share/pirut/pixmaps/puplet-downloading.png")
        self.trayicon = gtk.status_icon_new_from_pixbuf(pix)
        self.trayicon.set_from_pixbuf(pix)
        #self.trayicon = gtk.status_icon_new_from_stock(gtk.STOCK_COPY)
        self.trayicon.set_visible(True)
        self.trayicon.connect("popup_menu", self._Popup)
        self.trayicon.connect("activate", self._clicked)

    def _clicked(self, status):
        """ Tray icon is clicked.
            status: Well... Erm... I dunno what it is :)
        """
        def menu_pos(menu):
            return gtk.status_icon_position_menu(menu, self.trayicon)
        self.ui.TrayMenu1.popup(None, None, menu_pos, 0,
                                gtk.get_current_event_time())
        self.ui.TrayMenu1.show()

    def _Popup(self, status, button, time):
        """ Popup the menu at the right position """
        def menu_pos(menu):
            return gtk.status_icon_position_menu(menu, self.trayicon)
        
        self.ui.TrayMenu1.popup(None, None, menu_pos, button, time)
        self.ui.TrayMenu1.show()

    def _setupLogger(self, severity):
        """ Setup the fwbackups logger """
        if severity.lower() == 'debug':
            level = logging.DEBUG
        elif severity.lower() == 'info':
            level = logging.INFO
        elif severity.lower() == 'warning':
            level = logging.WARNING
        elif severity.lower() == 'error':
            level = logging.ERROR
        elif severity.lower() == 'critical':
            level = logging.CRITICAL
        else:
            raise ValueError(_('Invalid level \'%s\'') % level)
        try:
            logging.setLoggerClass(fwlogger.Logger)
            # Create logger and set level to debug
            log = logging.getLogger(LOGGERNAME)
            log.setLevel(level)
            # Create console & handler from console
            logconsole = guiwidgets.TextViewConsole(self.ui.LogViewerTextview)
            loghandler = guiwidgets.TextViewLogHandler(logconsole)
            # Create formatter & add formatter to handler
            logformatter = logging.Formatter("%(message)s")
            loghandler.setFormatter(logformatter)
            # add handler to logger
            log.addHandler(loghandler)
            log.connect(self.updateErrorWarnLabel)
            buffer = logconsole.buffer
            start = buffer.get_start_iter()
            end = buffer.get_end_iter()
            self.ui.ErrorsInLogLabel.set_text('0')
            self.ui.FailedBackupRunsLabel.set_text('0')
            for i in buffer.get_text(start, end).split('\n'):
                if i.startswith(_('ERROR')):
                    self.updateErrorWarnLabel('error',  i, False)
                elif i.startswith(_('WARNING')):
                    self.updateErrorWarnLabel('warning', i, False)
                if i.startswith(_('WARNING')) and i.endswith(_('Backup failed!')):
                    self.updateFailedBackupRunsLabel(self)
            return log, loghandler
        except:
            raise
            
    def _setDefaults(self):
        if USER != 'root':
            self.ui.SetDiskInfoToFileCheck.set_active(False)
            self.ui.SetDiskInfoToFileCheck.set_sensitive(False)
            self.ui.OneTimeDiskInfoToFileCheck.set_active(False)
            self.ui.OneTimeDiskInfoToFileCheck.set_sensitive(False)
        # Default to page 0...
        self.ui.AboutNotebook.set_current_page(0)
        self.ui.MainNotebook.set_current_page(0)
        self.ui.BackupDiaNotebook.set_current_page(0)
        # Default fields...
        self.ui.RestoreSetNameBox.set_active(0)
        self.ui.RestoreSetDateBox.set_active(0)
        # Default Labels...
        self._setLabels()
        # Default progressbars...
        self.SetView1.progressbar.set_text(_('Click \'Backup Set Now\' to begin'))
        self.OneTimeView1.progressbar.set_text(_('Click \'Backup\' to begin'))
        self.ui.RestoreProgress.set_text(_('Click \'Restore\' to begin'))
        # Don't show tabs on main...
        self.ui.MainNotebook.set_show_tabs(False)

def usage():
    """ Print the application usage """
    print _("""Usage:
-h, --help\tPrint this message and exit
-v, --verbose\tIncrease verbosity, enable debug messages
-b, --backup=NAME\tBackup set NAME now
-o, --onetime [PATHS]\tPerform a one-time backup. Will take all other arguments\n
                     \tand treat each as a path to backup.
""")

# Only if we're in main execution
if __name__ == "__main__":
    verbose = False
    try:
        avalableOptions = ["help", "verbose", "backup", "onetime"]
        # letter = plain options
        # letter: = option with arg
        (opts, rest_args) = getopt.getopt(sys.argv[1:],"hvb:o", avalableOptions)
    except (getopt.GetoptError), e:
        print e, ""
        usage()
        sys.exit(1)
    # Parse args, take action
    if opts == []:
        pass
    else:
        for (opt, value) in opts:
            if opt == "-h" or opt == "--help":
                usage()
                sys.exit(1)
            if opt == "-v" or opt == "--verbose":
                verbose = True
            """if opt == "-X" or opt == "--X":
                dostuffhere(value)"""

    try:
        # Startup the application and call the gtk event loop
        MainApp = fwbackups(verbose)
        gtk.main()
    except KeyboardInterrupt:
        # ctrl+c?
        MainApp.main_close(None)
